
'''
	import vessels.SSH.send as SSH_send
	
	from os.path import dirname, join, normpath
	import pathlib
	import sys

	this_directory = pathlib.Path (__file__).parent.resolve ()
	from_directory = normpath (join (this_directory, "../platform"))
	
	SSH_send.splendidly ({
		"private key": "/online ellipsis/vegan/DO/vegan/RSA.private",
		
		"from": {
			"directory": from_directory
		},
		
		"to": {
			"address": "164.92.112.195",
			"directory": "/platform"
		}
	})
'''
import os

def splendidly (parameters):
	private_key = parameters ["private key"]

	from_directory = parameters ["from"] ["directory"]
	
	to_directory = parameters ["to"] ["directory"]
	to_address = parameters ["to"] ["address"]

	script = " ".join ([
		"rsync -r -a --info=progress2 -v --delete",
		"--delete-excluded --progress --human-readable",
		f"""
			-e "ssh -o StrictHostKeyChecking=no -i '{ private_key }'"
		""".strip (),
		f'{ from_directory }/',
		f"root@{ to_address }:{ to_directory }"
	])

	print (script);

	os.system (script)




'''
	import vessels.proxies.HA.configs.HTTP_balancer as HA_HTTP_balancer
	HA_HTTP_balancer.build (
		from_port = "80",
		to_addresses = [
			"0.0.0.0:8000",
			"0.0.0.0:8001"
		]
	)
'''

'''
	http://localhost:8404/stats
'''


"""
	backend servers
		balance roundrobin
		server server1 0.0.0.0:8000 maxconn 32
		server server2 0.0.0.0:8001 maxconn 32
"""

def build (
	from_port = "80",
	to_addresses = [],
	
	#
	#	roundrobin
	#
	balance = "leastconn",
	
	config_path = ""
):	
	'''
		examples:
			server server1 0.0.0.0:8000 check
			server server1 0.0.0.0:8001 check
	'''
	site_number = 1
	backend_sites = ""
	for to_address in to_addresses:
		name = "site_" + str (site_number)
		backend_sites += f"\tserver { name } { to_address } check\n"		
		site_number += 1


	config = f"""
global
	daemon
	maxconn 256

defaults
	mode http
	timeout connect 5000ms
	timeout client 50000ms
	timeout server 50000ms

frontend stats
	bind *:8404
	stats enable
	stats uri /stats
	stats refresh 10s
	stats admin if LOCALHOST

frontend http-in
	bind *:{ from_port }
	default_backend site
	
backend site
	balance { balance }
{ backend_sites }
"""

	if (len (config_path) >= 1):
		FP = open (config_path, "w")
		FP.write (config)
		FP.close ()
		
		print ("The config was written to:", config_path)

	return config;




'''

import vessels
sels = vessels.prevail ({
	"port": "57384",
	"
})

app = sels.app;

@app.route ('/')
def home ():		
	return send_file (home_html)
'''

'''
	priorities:
		gunicorn
		gevent
'''


def clique ():
	print ("vessels clique")

from flask import Flask, request, send_file, request, make_response, send_from_directory, Response
from flask_cors import CORS
from flask_limiter import Limiter
from flask_limiter.util import get_remote_address
from flask_socketio import SocketIO, emit

def prevail (parameters):

	app = Flask (__name__)
	
	
	
	app.run (
		port = port
	)

	return;








def add_to_system_paths (trails):
	import pathlib
	from os.path import dirname, join, normpath
	import sys
	
	this_directory = pathlib.Path (__file__).parent.resolve ()
	for trail in trails:
		sys.path.insert (0, normpath (join (this_directory, trail)))

add_to_system_paths ([ 
	'structures',
	'structures_pip'
])


import os
import pathlib
from os.path import dirname, join, normpath
import sys
this_directory = pathlib.Path (__file__).parent.resolve ()

config_path = str (normpath (join (this_directory, "haproxy.conf")))
config_path = "/etc/haproxy/haproxy.cfg"


defaults_2 = f'''
defaults
	mode                    http
	log                     global
	option                  httplog
	option                  dontlognull
	option http-server-close
	option forwardfor       except 127.0.0.0/8
	option                  redispatch
	retries                 3
	
	# errorfile 503 /etc/haproxy/errors/503.http
	
	timeout http-request    10s
	timeout queue           1m
	timeout connect         10s
	timeout client          1m
	timeout server          1m
	timeout http-keep-alive 10s
	timeout check           10s
	
	maxconn                 3000
'''

backend = f'''
backend app
	balance leastconn
	#balance roundrobin
	server app10 0.0.0.0:8000 check
	server app20 0.0.0.0:8001 check
'''

backend = f'''
backend app
	server app_10 127.0.0.1:8000 check
'''


defaults = f'''
defaults
	mode http
	timeout connect 5000ms
	timeout client 50000ms
	timeout server 50000ms
'''

config = f"""
global
	chroot /var/lib/haproxy

	daemon
	maxconn 256
	user haproxy
	group haproxy

	# turn on stats unix socket
	# stats socket /var/lib/haproxy/stats

{ defaults }

frontend http-in
	bind *:80
	# mode http
	
	
	default_backend app

{ backend }
"""




import configs.HTTP_balancer as HTTP_balancer_config
config = HTTP_balancer_config.build ({
	"from": "80",
	"to": [
		"0.0.0.0:8000",
		"0.0.0.0:8001"
	]
})

FP = open (config_path, "w")
FP.write (config)
FP.close ()

os.system (f"haproxy -f '{ config_path }' -c")
os.system ("systemctl restart haproxy")
os.system (f"cat '{ config_path }'")
os.system ("systemctl status haproxy -l --no-pager")


#os.system ("haproxy -f haproxy.conf -c")
#os.system ("haproxy -f haproxy.conf")





'''
	priorities:
		import vessels.walls.UFW as UFW
		UFW.start ({
			"out": "yes",
			
			"ports": [{
				"number": 443,
				"protocols": [ "TCP" ],
				
				"out": "yes",
				"in": "yes"
			},{
				"number": 80,
				"protocols": [ "TCP" ],
				
				"out": "yes",
				"in": "yes"
			},{
				"number": 22,
				"protocols": [ "TCP", "UDP" ],
				
				"out": "yes",
				"in": "yes"
			}]
		})
'''

def start ():
	return;




'''
import vessels.proxies.HA.SSL as HA_SSL
HA_SSL.build_papers (
	certificate_path = "/etc/haproxy/SSL/certificate.pem",
	key_path = "certificate.pem.key"
)
'''

import os

def build_papers (
	certificate_path = "",
	key_path = "",
	
	days = "20000"
):

	assert (len (certificate_path) >= 1);
	assert (len (key_path) >= 1);
	
	script = " ".join ([
		"openssl req -x509 -newkey rsa:4096",
		f"-keyout '{ key_path }'",
		f"-out '{ certificate_path }'",
		f'-sha256 -days { days } -nodes -subj "/C=/ST=/L=/O=/OU=/CN="'
	])

	os.system (script)
	
	return;